# -*- coding: utf-8 -*-
# @Time     : 2021/6/3 17:34
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : recursive_example.py
# @info     :
import os.path


class EnglishRuler:
    def draw_line(self, tick_length, tick_label=''):
        """Draw one line with given tick length (followed by optional label)."""
        line = '-' * tick_length
        if tick_label:
            line += '  ' + tick_label
        print(line)

    def draw_interval(self, center_length):
        """Draw tick interval based upon a central tick length."""
        if center_length > 0:
            self.draw_interval(center_length - 1)
            self.draw_line(center_length)
            self.draw_interval(center_length - 1)

    def draw_ruler(self, num_inches, major_length):
        """Draw English ruler with given number of inches, major tick length"""
        self.draw_line(major_length, '0')
        for j in range(1, 1 + num_inches):
            self.draw_interval(major_length - 1)
            self.draw_line(major_length, str(j))


class Search:
    def __init__(self, data):
        self._data = data
        print(self._data[:10])

    def binary_search(self, target, low, high):
        if low > high:
            return False, -1
        else:
            mid = (low + high) // 2
            if target == self._data[mid]:
                return True, target
            elif target < self._data[mid]:
                return self.binary_search(target, low, mid - 1)
            else:
                return self.binary_search(target, mid + 1, high)


class FileSystem:
    def disk_usage(self, path):
        total = os.path.getsize(path)
        if os.path.isdir(path):
            for filename in os.listdir(path):
                childpath = os.path.join(path, filename)
                total += self.disk_usage(childpath)

        print('{0:<10}'.format(total), path)
        return total


if __name__ == '__main__':
    # english_ruler = EnglishRuler()
    # english_ruler.draw_ruler(3, 5)

    # search = Search([i for i in range(10000)])
    # print(search.binary_search(500, 0, 10000))

    fs = FileSystem()
    print(os.path.abspath(os.path.curdir))
    fs.disk_usage(os.path.curdir)



# -*- coding: utf-8 -*-
# @Time     : 2021/4/18 1:35
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : classify.py
# @info     : 本模块用于根据提取特征，对切片进行癌和非癌的分类,本模块支持二维数据的处理

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from pandas import DataFrame
import sklearn as sk
from sklearn.ensemble import RandomForestClassifier
from typing import List
from ufy.core.metrics._classification import Evaluation

try:
    # 老版本
    from sklearn import joblib
except:
    # 新版本
    import joblib
import os
import tensorflow as tf
from tensorflow.keras import layers
from xgboost import XGBClassifier
from tqdm import tqdm


def find_csv_in_dir(dirpath, allfiles=None):
    file_list = os.listdir(dirpath)
    for f in file_list:
        cur_f = os.path.join(dirpath, f)
        if os.path.isdir(cur_f):
            find_csv_in_dir(cur_f, allfiles)
        elif os.path.basename(f).split('.')[-1] == 'csv':
            allfiles.append(cur_f)


def load_data_csvs(filenames, target_cols=1, drop_id=True, id_cols=1, split_rate=0.7, shuffle=True):
    '''
    :param filename: csv数据文件名列表
    :param col_targets: 标签所在列的数量，可能是多标签,要求标签必须在最后几列
    :param drop_id: 是否删除id列
    :param id_endcols: id相关信息有多少列，默认前k列为id相关信息列
    :param split_rate: 换分比例，默认训练集占70%
    :return: 划分好的数据集(x_train,y_train),(x_test,y_test)
    '''
    data = []
    for filename in tqdm(filenames):
        print('load data:', filename)
        data_file = pd.read_csv(filename)
        data_file = np.array(data_file)
        if data == []:
            data = data_file
        else:
            data = np.vstack((data, data_file))
    if drop_id:
        data = data[:, id_cols:]
        data = data.astype('float32')

    if shuffle:
        # data = data[np.random.permutation(len(data))]
        np.random.shuffle(data)  # 功能与前行代码相同
    split_point = int(len(data) * split_rate)
    train_data = data[:split_point]
    test_data = data[split_point:]

    x_train, y_train = train_data[:, :-target_cols], train_data[:, -target_cols:]
    x_test, y_test = test_data[:, :-target_cols], test_data[:, -target_cols:]

    return (x_train, y_train.astype(int)), (x_test, y_test.astype(int))


def plot_roc_curve(y_trues, y_pres, model_names=None, pos_label=1, threshhold=0.5, issave=False, savename='roc.jpg',
                   analysis_filename='results_analysis.txt'):
    '''
    :param y_trues: 真实标签
    :param y_pres:  预测值
    :param pos_label: 正例标签
    :param issave: 是否保存结果
    :param savename: 保存结果图片的名
    :return: 绘制ROC曲线
    '''
    if model_names == None:
        model_names = [str(i + 1) for i in range(len(y_pres))]
    fp = open(analysis_filename, 'w', encoding='utf-8')
    for i in range(len(y_pres)):
        fpr, tpr, thresholds = sk.metrics.roc_curve(y_trues[i], y_pres[i], pos_label=pos_label)
        # print(fpr,'\n',tpr)
        roc_auc = sk.metrics.auc(fpr, tpr)
        # roc_auc = sk.metrics.roc_auc_score(y_trues[i], y_pres[i])
        print(model_names[i] + ':')
        y_pres_i = (y_pres[i] > threshhold).astype(int)
        print(sk.metrics.classification_report(y_trues[i], y_pres_i, digits=4))
        fp.write(model_names[i] + ':\n')
        fp.writelines(sk.metrics.classification_report(y_trues[i], y_pres_i, digits=4))
        fp.write('\n\n')
        plt.plot(fpr, tpr, label=model_names[i] + ' (AUC={0:.4f})'.format(roc_auc), lw=2)
    plt.plot([0, 1], [0, 1], color='navy', lw=2, linestyle='--')
    plt.xlim([-0.05, 1.05])
    plt.ylim([-0.05, 1.05])
    plt.xlabel('False Positive Rate')
    plt.ylabel('True Positive Rate')
    plt.title('ROC Curve')
    plt.legend(loc='lower right')

    if issave:
        plt.savefig(savename)  # 要在show之前保存

    plt.show()


def min_max_normal(x):
    '''
    :param x: 是一个二维数据
    :return: 一个单位化的数据
    '''
    min_col = np.min(x, axis=0)
    max_col = np.max(x, axis=0)
    return (x - min_col) / (max_col - min_col + 1e-7)


def softmax_temp(x, T=1):
    '''
    :param x: 带处理的数据
    :param T: 温度，默认温度为1
    :return: 根据温度，来进行softmax变换，并返回变换结果
    '''
    x = np.array(x)
    x_row_max = np.max(x, axis=-1)
    # x_row_max = x_row_max.reshape(list(x.shape)[:-1]+[1])
    x = x - x_row_max
    x_exp = np.exp(x / T)
    # x_exp_row_sum = x_exp.sum(axis=-1).reshape(list(x.shape)[:-1]+[1])
    x_exp_row_sum = x_exp.sum(axis=-1)
    softmax = x_exp / x_exp_row_sum
    return softmax


def MLP_softmax(x_train, y_train, model_savepath='MLP_softmax.h5', istraining=False, max_epochs=10000,
                batch_size=32, validation_split=0.3, verbose=1):
    if os.path.exists(model_savepath):
        model = tf.keras.models.load_model(model_savepath)
    else:
        istraining = True
        model = tf.keras.Sequential()
        model.add(layers.Dense(512, activation='relu', input_shape=(884,)))
        model.add(layers.Dropout(0.5))
        model.add(layers.Dense(256, activation='relu'))
        model.add(layers.Dense(128, activation='relu'))
        model.add(layers.Dense(64, activation='relu'))
        model.add(layers.Dropout(0.2))
        model.add(layers.Dense(2, activation='softmax'))

        model.summary()

    if istraining:
        print(y_train.shape)
        y_train_onehot = tf.one_hot(y_train, depth=2)
        print(y_train_onehot.shape)
        y_train_onehot = tf.reshape(y_train_onehot, (-1, 2))
        print(y_train_onehot.shape)

        model.compile(optimizer=tf.keras.optimizers.Adam(),
                      loss=tf.keras.losses.categorical_crossentropy,
                      metrics=['acc'])
        callback_list = [tf.keras.callbacks.EarlyStopping(monitor='acc', patience=8),
                         tf.keras.callbacks.ModelCheckpoint(filepath=model_savepath, monitor='val_loss',
                                                            save_best_only=True)]
        model.fit(x_train, y_train_onehot, batch_size=batch_size, epochs=max_epochs, validation_split=validation_split,
                  verbose=verbose, callbacks=callback_list)

    return model


def MLP_sigmoid(x_train, y_train, model_savepath='MLP_softmax.h5', istraining=False, max_epochs=10000,
                batch_size=32, validation_split=0.3, verbose=1):
    if os.path.exists(model_savepath):
        model = tf.keras.models.load_model(model_savepath)
    else:
        istraining = True
        model = tf.keras.Sequential()
        model.add(layers.Dense(512, activation='relu', input_shape=(884,)))
        model.add(layers.Dropout(0.5))
        model.add(layers.Dense(256, activation='relu'))
        model.add(layers.Dense(128, activation='relu'))
        model.add(layers.Dense(64, activation='relu'))
        model.add(layers.Dropout(0.2))
        model.add(layers.Dense(1, activation='sigmoid'))

        model.summary()

    if istraining:
        model.compile(optimizer=tf.keras.optimizers.Adam(),
                      loss=tf.keras.losses.binary_crossentropy,
                      metrics=['acc'])
        callback_list = [tf.keras.callbacks.EarlyStopping(monitor='acc', patience=8),
                         tf.keras.callbacks.ModelCheckpoint(filepath=model_savepath, monitor='val_loss',
                                                            save_best_only=True)]
        model.fit(x_train, y_train, batch_size=batch_size, epochs=max_epochs, validation_split=validation_split,
                  verbose=verbose, callbacks=callback_list)

    return model


def KNN(x_train, y_train, model_savepath='knn.pkl', istraining=False):
    if os.path.exists(model_savepath):
        model = joblib.load(model_savepath)
    else:
        istraining = True
        model = sk.neighbors.KNeighborsClassifier()
    if istraining:
        model.fit(x_train, y_train)
        joblib.dump(model, model_savepath)
    return model


def RandomForest(x_train, y_train, model_savepath='RandomForest.pkl', istraining=True):
    if os.path.exists(model_savepath):
        model = joblib.load(model_savepath)
    else:
        istraining = True
        model = RandomForestClassifier()
    if istraining:
        model.fit(x_train, y_train)
        joblib.dump(model, model_savepath)
    return model


def XGboost(x_train, y_train, model_savepath='xgboost.pkl', istraining=True):
    if os.path.exists(model_savepath):
        model = joblib.load(model_savepath)
    else:
        istraining = True
        model = XGBClassifier()
    if istraining:
        model.fit(x_train, y_train, verbose=True)
        # model.save_model(model_savepath)
        joblib.dump(model, model_savepath)
    return model


def model_test(x_test, x_test_normal, y_test, threshhold: float,models_path: List, models_name: List, savename: str,
               analysis_filename: str,
               summaryname: str):
    y_trues = [y_test for i in range(len(models_name))]
    y_preds = []
    for i in tqdm(range(len(models_path))):

        # input
        if 'MLP' in models_name[i] or 'normal' in models_name[i]:
            x = x_test_normal
        else:
            x = x_test

        # model
        if 'MLP' in models_name[i]:
            model = tf.keras.models.load_model(models_path[i])
        else:
            model = joblib.load(models_path[i])

        # predict
        if models_name[i] == 'MLP sigmoid':
            y_pred = model.predict_proba(x)[:, 0]
        else:
            y_pred = model.predict_proba(x)[:, 1]
        y_preds.append(y_pred)

    plot_roc_curve(y_trues, y_preds, models_name, threshhold=threshhold, pos_label=1, issave=True, savename=savename,
                   analysis_filename=analysis_filename)

    summary_data = DataFrame(columns=['model', 'Accuracy', 'Kappa', 'sensitivity', 'Specificity', 'F1-score', 'AUC'])
    for i in range(len(models_name)):
        evaluate = Evaluation(model_name=models_name[i], y_true=y_trues[i], y_pred=y_preds[i], lables=[1, 0],thresh_hold=threshhold)
        summary_data.loc[i] = [evaluate.model_name, evaluate.accuracy, evaluate.kappa, evaluate.sensitivity,
                               evaluate.specificity, evaluate.f1_score, evaluate.auc]
    if not summaryname.endswith('.csv'):
        raise ValueError('请使用csv文件名！！！')
    summary_data.to_csv(summaryname)


if __name__ == '__main__':
    allfiles = []
    find_csv_in_dir('../data/diagnosis/new_cancer/', allfiles)
    print(allfiles)



# -*- coding: utf-8 -*-
# @Time     : 2021/5/13 17:01
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : scaler.py
# @info     : 本模块集成各种归一化或标准化方法

from ufy.np import np

'''
'''
class Standard_scaler():
    def __init__(self):
        self.__mean = None
        self.__std = None

    def fit(self, X: np.ndarray, axis: tuple):
        self.__mean = np.mean(X, axis=axis)
        self.__std = np.std(X, axis=axis)

    def transform(self, X: np.ndarray):
        assert self.__mean is not None and self.__std is not None, 'must fit before transform!'

        epsilon = 1e-7
        out = (X - self.__mean) / (self.__std + epsilon)
        return out


class Min_max_scaler():
    def __init__(self):
        self.__min = None
        self.__std = None

    def fit(self, X: np.ndarray, axis: tuple):
        self.__min = np.min(X, axis=axis)
        self.__max = np.max(X, axis=axis)

    def transform(self, X: np.ndarray):
        assert (self.__min is not None) and (self.__max is not None), 'must fit before transform!'

        epsilon = 1e-7
        out = (X - self.__min) / (self.__max - self.__min + epsilon)
        return out



import os

from PIL import Image
import numpy as np
from scipy import ndimage as ndi
import cv2
from typing import List


def energy(img, savename='energy.jpg', issave=False):
    """
        Simple gradient magnitude energy map.
    """
    xgrad = ndi.convolve1d(img, np.array([1, 0, -1]), axis=1, mode='wrap')
    ygrad = ndi.convolve1d(img, np.array([1, 0, -1]), axis=0, mode='wrap')

    grad_mag = np.sqrt(np.sum(xgrad ** 2, axis=2) + np.sum(ygrad ** 2, axis=2))

    if issave:
        # cv2.imwrite(savename, grad_mag.astype(np.uint8))
        image = Image.fromarray(grad_mag.astype(np.uint8))
        image.save(savename)
    return grad_mag


def find_file(dir_source: str, file_format: List[str]) -> List[str]:
    ''' 在源文件夹，递归搜索file_format 格式的文件，并用file_list返回
    :param dir_source: 源文件夹
    :param file_format: 需要搜索的文件格式
    :return: 返回符合格式的文件的列表：file_list
    '''
    dirlist = os.listdir(dir_source)

    file_list = []
    for f in dirlist:
        f = os.path.join(dir_source, f)
        if os.path.isdir(f):
            file_list += find_file(f, file_format)
        elif str.lower(os.path.basename(f).split('.')[-1]) in file_format:
            file_list.append(f)
    return file_list


if __name__ == '__main__':
    # alllist = []
    alllist = find_file("E:/OneDrive/rebar/一期善后/stitich", file_format=['json'])
    print(alllist)



# -*- coding: utf-8 -*-
# @Time     : 2021/5/8 23:17
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : BinTree.py
# @info     :

class BiTreeNode:
    def __init__(self, val):
        self.data = val
        self.left = None
        self.right = None


class BinTree:
    def __init__(self, rootval):
        self.root = BiTreeNode(rootval)
        pass

    def add(self, value, parent: BiTreeNode, child):
        if child == 'left':
            parent.left = BiTreeNode(value)
        elif child == 'right':
            parent.right = BiTreeNode(value)
        else:
            raise ValueError('指定孩子类型有误!!!,二叉树只有left,right两种孩子类型，没有'+child+'类型')


    def remove(self, node: BiTreeNode):
        pass

    def traverse(self, way: str = 'prior_recursive'):
        pass

    def __traverse_prior__(self):
        '''
        :return:
        '''
        pass

if __name__ == '__main__':
    bitree = BinTree(0)
    bitree.add(1,bitree.root,'left')



# -*- coding: utf-8 -*-
# @Time     : 2021/4/22 19:13
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : pca.py
# @info     :

import numpy as np
def PCA(dataMat, topNfeat=9999999):
    '''
    :param dataMat: 待压缩的数据矩阵
    :param topNfeat: 压缩后的维度
    :return: 降维后的低维数据，原始数据的重构数据（一般与原始数据不同，有一定的信息损失）
    '''
    meanVals = np.mean(dataMat,axis=0)
    meanRemoved = dataMat - meanVals
    covMat = np.cov(meanRemoved,rowvar=0)
    eigVals, eigVects = np.linalg.eig(np.mat(covMat))
    eigValInd = np.argsort(eigVals)
    eigValInd = eigValInd[:-(topNfeat+1):-1]
    redEigVects = eigVects[:,eigValInd]
    lowDDataMat  = meanRemoved * redEigVects
    reconMat = (lowDDataMat * redEigVects.T) + meanVals
    return lowDDataMat,reconMat

def PCA2(X):
    '''
    主成分分析
    :param X: 矩阵X，其中该矩阵中存储训练数据，每一行为一条训练数据
    :return: 投影矩阵（按照维度的重要性排序）、方差和均值
    '''
    # 获取维数
    num_data, dim = X.shape
    # 数据中心化
    mean_X = X.mean(axis=0)
    X = X - mean_X


    if dim > num_data:
        # PCA- 使用紧致技巧
        M = np.dot(X, X.T)  # 协方差矩阵
        e, EV = np.linalg.eigh(M)  # 特征值和特征向量
        tmp = np.dot(X.T, EV).T  # 这就是紧致技巧
        V = tmp[::-1]  # 由于最后的特征向量是我们所需要的，所以需要将其逆转
        S = np.sqrt(e)[::-1]  # 由于特征值是按照递增顺序排列的，所以需要将其逆转
        for i in range(V.shape[1]):
            V[:, i] /= S
    else:
        # PCA- 使用 SVD 方法
        U, S, V = np.linalg.svd(X)
        V = V[:num_data]  # 仅仅返回前 nun_data 维的数据才合理
        # 返回投影矩阵、方差和均值
    return V, S, mean_X



# -*- coding: utf-8 -*-
# @Time     : 2021/6/16 10:54
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : _classification.py
# @info     :


from sklearn.metrics import confusion_matrix, roc_curve, auc
import numpy as np


class Evaluation:
    def __init__(self, model_name, y_true, y_pred, thresh_hold, lables=None, pos_label=1):
        self.model_name = model_name

        # sklearn.metrics.confusion_matrix()
        def continuous2binary(y_pred, threshold):
            out = np.zeros_like(y_pred)
            out[y_pred > threshold] = 1
            return out

        y_pred_ = continuous2binary(y_pred, thresh_hold)
        self.confusion_matrix = confusion_matrix(y_true=y_true, y_pred=y_pred_, labels=lables)
        _fpr, _tpr, thresholds = roc_curve(y_true, y_pred, pos_label=pos_label)
        self.auc = auc(_fpr, _tpr)
        self.kappa = kappa(self.confusion_matrix)
        self.sensitivity = sensitivity(self.confusion_matrix)
        self.specificity = specificity(self.confusion_matrix)
        self.accuracy = accuracy(self.confusion_matrix)
        self.precision = precision(self.confusion_matrix)
        self.recall = recall(self.confusion_matrix)
        self.f1_score = 2 * (self.precision * self.recall) / (self.precision + self.recall)

    def summary(self):
        print()


def kappa(c_matrix):
    '''
    :param c_matrix: 混淆矩阵
    :return: kappa 系数
    '''

    N = np.sum(c_matrix)
    ground_truth = np.sum(c_matrix, axis=0)  # 真值
    pre = np.sum(c_matrix, axis=1)  # 预测值

    po = np.trace(c_matrix) / N
    pe = np.dot(ground_truth, pre) / N ** 2

    kappa_coef = (po - pe) / (1 - pe)
    return kappa_coef


def sensitivity(c_matrix):
    '''
    :param c_matrix: 混淆矩阵
    :return: 灵敏度[TPR]
    '''
    assert np.array(c_matrix).shape == (2, 2)
    TP = c_matrix[0][0]
    FN = c_matrix[0][1]
    FP = c_matrix[1][0]
    TN = c_matrix[1][1]

    tpr = TP / (TP + FN)
    # tnr = TN / (FP + TN)
    return tpr


def specificity(c_matrix):
    '''
    :param c_matrix: 混淆矩阵
    :return: 特异度[TNR]
    '''
    assert np.array(c_matrix).shape == (2, 2)
    TP = c_matrix[0][0]
    FN = c_matrix[0][1]
    FP = c_matrix[1][0]
    TN = c_matrix[1][1]

    # tpr = TP / (TP + FN)
    tnr = TN / (FP + TN)
    return tnr


def accuracy(c_matrix):
    '''
    :param c_matrix: 混淆矩阵
    :return: acc
    '''
    return np.trace(c_matrix) / np.sum(c_matrix)


def precision(c_matrix):
    '''
    :param c_matrix:
    :return:
    '''
    assert np.array(c_matrix).shape == (2, 2)
    TP = c_matrix[0][0]
    FN = c_matrix[0][1]
    FP = c_matrix[1][0]
    TN = c_matrix[1][1]

    precise = TP / (TP + FP)
    return precise


def recall(c_matrix):
    assert np.array(c_matrix).shape == (2, 2)
    TP = c_matrix[0][0]
    FN = c_matrix[0][1]
    FP = c_matrix[1][0]
    TN = c_matrix[1][1]

    rc = TP / (TP + FN)
    return rc


if __name__ == '__main__':
    y_true = np.random.randint(2, size=1000)
    y_pred = np.random.randint(2, size=1000)
    print(y_true)
    print(y_pred)
    e = Evaluation(model_name='test', y_true=y_true, y_pred=y_pred, pos_label=1, lables=[1, 0])
    print(e.confusion_matrix)
    print(e.auc)
    print(e.accuracy)
    print(e.recall)
    print(e.precision)



# -*- coding: utf-8 -*-
# @Time     : 2021/5/24 17:22
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : udata.py
# @info     :

import pandas as pd
from pandas import DataFrame


class UData:
    def read(self, datafile: str) -> None:
        if datafile.endswith('.xlsx'):
            self.data = pd.read_excel(datafile)
        elif datafile.endswith('.csv'):
            self.data = pd.read_csv(datafile)
        else:
            self.data = DataFrame()
            raise ValueError('This methods is only support ".xlsx" and ".csv" files, please check your filename again.')

    def save(self, data: DataFrame, savename: str, index=False) -> None:
        if savename.endswith('.xlsx'):
            data.to_excel(savename, index=index)
        elif savename.endswith('.csv'):
            data.to_csv(savename, index=index)
        else:
            raise ValueError(
                'Can not save, please check your savename is correct. We just support ".xlsx" and ".csv" file.')



# -*- coding: utf-8 -*-
# @Time     : 2021/5/10 15:17
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : cutImage.py
# @info     :

from ufy.CV.cutImg import cut_img_energy,cut_img_dir


# -*- coding: utf-8 -*-
# @Time     : 2021/6/15 17:41
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : heatmap.py
# @info     :

# -*- coding: utf-8 -*-
# @Time     : 2021/6/1 10:32
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     :show.py
# @info     :


import seaborn as sns
from typing import Tuple
from ufy.HX.diagnosis.classify import *
from ufy.core.utils.tools import energy, find_file
from tqdm import tqdm
import os
from PIL import Image
import openslide
from matplotlib.colors import LinearSegmentedColormap


def min_max_normal(x):
    '''
    :param x: 是一个二维数据
    :return: 一个单位化的数据
    '''
    min_col = np.min(x)
    max_col = np.max(x)
    return (x - min_col + 1e-9) / (max_col - min_col + 1e-7)


def load_model(model_path):
    return joblib.load(model_path)


def load_data(tiles_fold: str, data_path: str):
    if data_path.endswith('xlsx'):
        data = pd.read_excel(data_path)
    elif data_path.endswith('csv'):
        data = pd.read_excel(data_path)
    else:
        raise ValueError('不支持该格式的数据文件')

    data = np.array(data)
    x_test = data[:, 1:885]
    x_test = np.array(x_test).astype(float)
    imgfiles = data[:, 0]
    imgfiles = tiles_fold + imgfiles
    return imgfiles, x_test


def predict(model_path, x_test):
    model = load_model(model_path)
    return model.predict_proba(x_test)[:, 1]


def grid_split(arr, grid=(1, 1)):
    out = [[None for j in range(grid[1])] for i in range(grid[0])]
    shape = arr.shape
    w_step, h_step = shape[1] // grid[1], shape[0] // grid[0]
    if shape[0] % grid[0] != 0 or shape[1] % grid[1] != 0:
        raise ValueError('Please input correct grid.')
    for i in range(grid[0]):
        for j in range(grid[1]):
            out[i][j] = arr[i * h_step:(i + 1) * h_step, j * w_step:(j + 1) * w_step]
    return np.array(out)


def grid_merge(arr):
    out = [np.hstack(arr[i, :, ...]) for i in range(arr.shape[0])]
    out = np.vstack(out)
    print('energy_matrix shape:', out.shape)
    return np.array(out)


def create_energy_matrix(imgfiles, savename, shape=(92, 44), grid=(1, 1)):
    print(grid)
    if os.path.exists(savename):
        return np.loadtxt(savename)
    energy_matrix = [[None for j in range(shape[1])] for i in range(shape[0])]
    for imgf in tqdm(imgfiles):
        img = np.array(Image.open(imgf))
        img[img == 0] = 255
        imgf = imgf.strip('.jpg')
        loc = imgf.split('_')
        i = int(loc[2]) // 2000
        j = int(loc[3]) // 2000
        energy_split = grid_split(energy(img), grid=grid)
        grid_en = [[np.mean(energy_split[i][j]) for j in range(grid[1])] for i in range(grid[0])]
        energy_matrix[i][j] = np.array(grid_en)

    print(energy_matrix)
    energy_matrix_ = np.array(energy_matrix)
    print(energy_matrix_.shape)
    out = grid_merge(energy_matrix_)
    energy_matrix_grid = min_max_normal(out)
    np.savetxt(savename, energy_matrix_grid)
    return energy_matrix_grid


def create_probablity_matrix(model_path, x_test, imgfiles, savename, shape=(92, 44), grid=(1, 1)):
    if os.path.exists(savename):
        return np.loadtxt(savename)
    model = load_model(model_path)
    y_pres = model.predict_proba(x_test)[:, 1]
    probablity_matrix = np.zeros(shape=shape)
    for i, imgf in enumerate(imgfiles):
        imgf = imgf.strip('.jpg')
        loc = imgf.split('_')
        x_loc = int(loc[2]) // 2000
        y_loc = int(loc[3]) // 2000
        probablity_matrix[x_loc][y_loc] = y_pres[i]
    image = Image.fromarray(probablity_matrix)
    image = image.resize((shape[1] * grid[1], shape[0] * grid[0]))
    probablity_matrix = np.array(image)
    np.savetxt(savename, probablity_matrix)
    return probablity_matrix


def show_img(imgfiles_dir, shape: Tuple, savename: str):
    imgs = [[None for j in range(shape[1])] for i in range(shape[0])]
    imgfiles = os.listdir(imgfiles_dir)
    for imgf in imgfiles:
        imgf_str = imgf.strip('.jpg')
        loc = imgf_str.split('_')
        x_loc = int(loc[2]) // 2000
        y_loc = int(loc[3]) // 2000
        imgs[x_loc][y_loc] = plt.imread(os.path.join(imgfiles_dir, imgf))
    for i in range(shape[0] * shape[1]):
        plt.subplot(shape[0], shape[1], i)
        plt.imread(imgs[i])
    plt.show()


def show_heatmap(heatmatrix, savename: str, cmap='rainbow'):
    sns.heatmap(heatmatrix, cmap=cmap, square=True)
    plt.savefig(savename + '_heatmap.jpg', dpi=600)
    plt.show()


def show_heatmap_img(heatmatrix, background_file, savename: str):
    plt.figure(figsize=(23, 11))
    img = plt.imread(background_file)
    fig, ax = plt.subplots()
    ax.imshow(img)
    sns.heatmap(heatmatrix, cmap='rainbow', square=True)
    plt.savefig(savename + 'heatmap_img.jpg', dpi=600)


def thumbnail(slides, savedir):
    # 保存缩略图
    for f in tqdm(slides):
        source = openslide.open_slide(f)
        img = np.array(source.read_region((0, 0), 4, source.level_dimensions[4]).convert('RGB'))
        image = Image.fromarray(img).convert(mode='RGB')
        savename1 = savedir + f.split('/')[-1].replace('.mrxs', '') + '_black.jpg'
        image.save(savename1)

        img[img == 0] = 255
        image = Image.fromarray(img).convert(mode='RGB')
        savename2 = savedir + f.split('/')[-1].replace('.mrxs', '') + '_white.jpg'
        image.save(savename2)


def main(model_path, data_path, tiles_fold, shape: Tuple, savename: str, grid: Tuple = (1, 1)):
    imgfiles, x_test = load_data(tiles_fold, data_path)  # imgfiles 存放有特征值的小瓦块的文件名
    all_tiles = find_file(dir_source=tiles_fold, file_format=['jpg'])  # all_tiles 存放所有的小瓦块的文件名
    energy_matrix_grid = create_energy_matrix(
        all_tiles, savename=savename + 'energy.txt', shape=shape, grid=grid)
    probablity_matrix_grid = create_probablity_matrix(
        model_path=model_path, x_test=x_test, imgfiles=imgfiles, savename=savename + '_probability.txt', shape=shape,
        grid=grid)

    eh_matrix = energy_matrix_grid * 0.3 + probablity_matrix_grid
    show_heatmap(min_max_normal(eh_matrix), savename=savename + '_prob_en_map.jpg')

    overlay = energy_matrix_grid * 0.3 + probablity_matrix_grid
    m = overlay.max()
    overlay[overlay < 0.01] = 2 * m

    my_colormap = LinearSegmentedColormap.from_list("normal-tumor-white", ["purple", "cyan", "yellow"])
    show_heatmap(min_max_normal(overlay), savename=savename + '_overlay_map.jpg', cmap=my_colormap)



# -*- coding: utf-8 -*-
# @Time     : 2021/5/12 11:52
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : data_map.py
# @info     : 本模块用于处理文件映射： 映射关系： sickNo <- pathologyNum（图片信息） <- 片子名

# note      : 病理号格式：pre+year+num+subnum，其中主号包括pre+year+num
import datetime
import os
import re
import shutil

import pandas as pd
from pandas import DataFrame
import numpy as np
from tqdm import tqdm

default_pre = 'Z'

from typing import List


def clear_str(s: str, drop_char: List[str] = ['.']):
    for c in drop_char:
        s = s.replace(c, '')
    return s


def check_and_getsubNum(s, default_pre='Z', num_bit=5, drop_char=['.']):
    ''' 判断病理号是否合理并，并返回病理主号+副号
        :param s: 病理号字符串
        :param default_pre: 默认前缀，当字符串没有前缀时，默认前缀为Z
        :param num_bit: 编号位数限制（默认不能超过5位）
        :param drop_char: 需要去除的无效字符
        :return:
    '''
    if not isinstance(s, str) or (s == ''):
        return False, []

    s = s.upper()  # 统一处理为大写
    s = clear_str(s, drop_char=drop_char)  # 统一处理干扰字符

    pre, year, num, subnum = '', '', '', ''
    i = 0

    # 获取前缀信息：pre
    if not str.isalpha(s[0]):
        pre = default_pre
    else:
        while (i < len(s)) and str.isalpha(s[i]):
            pre += s[i]
            i += 1

    # 获取年份信息：year
    if len(s) - i < 3:
        return False, []
    if str.isdigit(s[i:i + 2]):
        year = s[i:i + 2]
        i += 2
    elif str.isdigit(s[i]):
        year = s[i]
        i += 1
    else:
        return False, []

    # 获取编号：num
    while (i < len(s)) and (not str.isdigit(s[i])):
        i += 1
    while (i < len(s)) and (str.isdigit(s[i])):
        num += s[i]
        i += 1
    if len(num) > num_bit:  # 超出位数，为不规范的编号
        return False, []

    # 获取副号：subnum
    while (i < len(s)) and (not str.isdigit(s[i])):
        i += 1
    while (i < len(s)) and (str.isdigit(s[i])):
        subnum += s[i]
        i += 1

    # print(pre, year, num, subnum)
    if subnum == '':
        return True, [pre, int(year), int(num), None]
    else:
        return True, [pre, int(year), int(num), int(subnum)]


def check_and_getmainNum(s, default_pre='Z', num_bit=5, drop_char=['.']):
    ''' 判断病理号是否合理并，并返回病理主号
    :param s: 病理号字符串
    :param default_pre: 默认前缀，当字符串没有前缀时，默认前缀为Z
    :param num_bit: 编号位数限制（默认不能超过5位）
    :param drop_char: 需要去除的无效字符
    :return:
    '''

    flag, subNum = check_and_getsubNum(s, default_pre, num_bit, drop_char)

    if not flag:
        return flag, []
    else:
        return flag, subNum[:3]


def check_and_getmainNum_multi(s, default_pre='Z', num_bit=5, drop_char=['.']) -> (bool, List):
    ''' 判断病理号是否合理并，并返回病理主号(多个病理号)
    :param s: 病理号字符串,可能含多个病理号
    :param default_pre: 默认前缀，当字符串没有前缀时，默认前缀为Z
    :param num_bit: 编号位数限制（默认不能超过5位）
    :param drop_char: 需要去除的无效字符
    :return:
    '''
    if not isinstance(s, str):
        return False, []
    else:
        arr = re.split('、|/|,', s)
        check = False
        mainNumbers = []
        for a in arr:
            flag, num = check_and_getmainNum(a, default_pre, num_bit, drop_char=drop_char)
            check |= flag
            if flag:
                mainNumbers.append(num)
        return check, mainNumbers


def match(s1, s2, default_pre='Z', num_bit=5, drop_char=['.'], way='main_num') -> bool:
    ''' 判断字符串s1 和 s2 是否匹配
    :param s1: 片子病理号，s1只可能包含一个病理号
    :param s2: 临床或病理表的的病理号，s2可能有多个病理号
    :param default_pre: 病理号，缺省前缀
    :param num_bit: 编号位数限制
    :param drop_char: 需要除去的无效字符
    :param way: 匹配方式，main_num, 根据主号匹配      sub_num, 根据主号和副号匹配
    :return: 是否匹配，True or False
    '''
    if way == 'main_num':
        flag1, num1 = check_and_getmainNum(s1, default_pre=default_pre, num_bit=num_bit, drop_char=drop_char)
        flag2, nums2 = check_and_getmainNum_multi(s2, default_pre=default_pre, num_bit=num_bit, drop_char=drop_char)
        if flag1 and flag2 and (num1 in nums2):
            return True
        else:
            return False

    if way == 'sub_num':
        flag1, num1 = check_and_getsubNum(s1, default_pre=default_pre, num_bit=num_bit, drop_char=drop_char)
        flag2, num2 = check_and_getsubNum(s2, default_pre=default_pre, num_bit=num_bit, drop_char=drop_char)
        if flag1 and flag2 and (num1 == num2):
            return True
        else:
            return False


def load_data_excel(file_name, col_name=['病理号'], sheet_names=None):
    '''
    :param file_name: 待读取的文件名
    :param col_name: 读取的列名
    :param sheet_names: 读取的sheet名
    :return: 返回指定sheet，col_name的结果数据，会将多个sheet的结果合并到一起
    '''
    print("load data:", file_name)
    rdexcle = pd.read_excel(file_name, sheet_name=sheet_names, usecols=col_name)
    output = []
    for e in rdexcle:
        output += np.array((rdexcle[e])).tolist()
    return output


def pathologyNum_mapto_sickNum(file_names1=[], file_names2=[], default_pre='Z', num_bit=5, col_name1=['病理号'],
                               sheet_names1=None, col_name2=['患者代码', '病理编号'], sheet_names2=None, savedir='') -> List[
    List]:
    '''
    :param file_names1: 病理图编号的文件集合（含带副号的病理编号）
    :param file_names2: 临床表文件集合（含病理编号和患者代码）
    :param default_pre: 默认病理号前缀
    :param num_bit: 病理号编号部分的位数，默认5位
    :param col_name1: 文件集合1要读取的列的集合
    :param col_name2: 文件集合2要读取的列的集合
    :param savename: 映射结果的保存文件名，默认为pathology_mapto_sickno.csv， 第一列为：病理图的病理编号，第二列为：患者代码
    :param show_file_name: 是否显示文件名，用于表明当前数据的来源
    :return: 保存结果文件，output为[病理图编号,sickno,病理主号,是否匹配到]的格式的list
    '''
    input1 = []
    names1 = []
    input2 = []
    names2 = []
    for f1 in file_names1:
        f1_data = load_data_excel(f1, col_name=col_name1, sheet_names=sheet_names1)
        input1 += f1_data
        names1 += [os.path.basename(f1) for i in range(len(f1_data))]

    for f2 in file_names2:
        f2_data = load_data_excel(f2, col_name=col_name2, sheet_names=sheet_names2)
        input2 += f2_data
        names2 += [os.path.basename(f2) for i in range(len(f2_data))]

    print(len(input1), len(input2))

    output = []
    count = 0
    for i in tqdm(range(len(input1))):
        line1 = input1[i]
        s1 = line1[0]
        s1_file = names1[i]
        flag = False
        for j, line2 in enumerate(input2):
            s2 = line2[1]
            s2_file = names2[j]
            if match(s1, s2, default_pre, num_bit, way='main_num'):
                output += [[s1, line2[0], s2, True, s1_file, s2_file]]  # sickno: line2[0]
                count += 1
                if count % 500 == 0:
                    print(count)
                flag = True
                break
        if not flag:
            output += [[s1, None, None, False, None, None]]
    print(count)

    out_data = DataFrame(columns=['病理号', '患者代码', '病理编号', '是否匹配', '病理号所在文件', '患者代码和病理编号所在文件'])

    for i in tqdm(range(len(output))):
        out_data.loc[i] = output[i]

    # 保存文件
    savename = os.path.join(savedir, 'pNum_mapto_sickNum_' + str(datetime.date.today()) + '.xlsx')
    out_data.to_excel(savename)

    return output


def copy_tiles_file(source_dir, dist_dir, listfile: str, usecols=['病理号']):
    ''' 根据文件的list，将数据匹配的数据重一个文件从source_dir 拷贝到 dist_dir
    :param source_dir:
    :param dist_dir:
    :param listfile: 提供list的文件，支持csv和xlsx文件
    :param usecols:
    :return:
    '''
    if listfile.endswith('.csv'):
        list_pathology = pd.read_csv(listfile, usecols=usecols)
    elif listfile.endswith('.xlsx'):
        list_pathology = pd.read_excel(listfile, usecols=usecols)
    else:
        raise ValueError('参数listfile，只支持csv，xlsx文件')
    listdir_source = os.listdir(source_dir)
    list_pathology = np.array(list_pathology).flatten().tolist()

    count = 0
    count_null = 0
    for pathology in tqdm(list_pathology):
        if not isinstance(pathology, str):
            print('空病理号')
            count_null += 1
            continue

        for sub_dir in listdir_source:
            if os.path.exists(os.path.join(dist_dir, sub_dir)):
                count += 1
                continue
            if check_and_getsubNum(pathology) == check_and_getsubNum(sub_dir):
                print(sub_dir)
                shutil.copytree(os.path.join(source_dir, sub_dir), os.path.join(dist_dir, sub_dir))
                count += 1
    print('总数：', len(list_pathology), '成功拷贝：', count, '空病理号：', count_null, '其它：',
          len(list_pathology) - count - count_null)



# -*- coding: utf-8 -*-
# @Time     : 2021/5/17 13:23
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : tools.py
# @info     :

from typing import List

from tqdm import tqdm
import pandas as pd
from pandas import DataFrame

from udata import UData


class Data(UData):
    def __init__(self, datafile: str) -> None:
        print('load data:', datafile)
        self.data = self.read(datafile)

    def select_data_by_list(self, choose_list: List, choose_col: str, savename='select.xlsx',
                            index_save: bool = False) -> (DataFrame, List):
        '''
        :param choose_list: 需要筛选的list
        :param choose_col: 筛选列
        :param savename: 结果保存名
        :param index_save: 是否保存index 列
        :return: 筛选后的的结果和没有找到的list
        '''
        choose_index = []
        finds = []
        for i in tqdm(range(len(self.data))):
            if self.data.loc[i, choose_col] in choose_list:
                choose_index.append(i)
                finds.append(self.data.loc[i, choose_col])
                choose_list.remove(self.data.loc[i, choose_col])

        notfinds = []
        for item in choose_list:
            if item not in finds:
                notfinds.append((item))

        out = self.data.loc[choose_index]
        self.save(data=out, savename=savename, index=index_save)
        return out, notfinds



# -*- coding: utf-8 -*-
# @Time     : 2021/4/22 19:36
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : setup.py.py
# @info     :

from setuptools import setup

__version__ = '0.1.12'

setup(name='ufy',
      version=__version__,
      description='some tools for ufy',
      author='ufy',
      author_eamil='antarm@outlook.com',
      url='https://github.com/antarm/ufy',
      packages=['ufy',
                'ufy.core','ufy.core.DataStructure', 'ufy.core.ML', 'ufy.core.DL', 'ufy.core.ML.compaction', 'ufy.core.preprocessing','ufy.core.utils','ufy.core.metrics',
                'ufy.CV',
                'ufy.datasets',
                'ufy.HX', 'ufy.HX.data_process','ufy.HX.diagnosis'])



# -*- coding: utf-8 -*-
# @Time     : 2021/4/26 18:16
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : cutImg.py
# @info     :


import os

# os.add_dll_directory('D:\Anaconda3\Library\openslide-win64-20171122\\bin')  # 添加自己的openslide动态库的文件路径
# os.add_dll_directory('D:\Anaconda3\Library\openslide-win64-20171122\\lib')  # 添加自己的openslide动态库的文件路径

##########################################################################################################################
# os.environ['PATH'] = 'D:\Anaconda3\Library\openslide-win64-20171122\\bin'+';'+os.environ['PATH']   in lowlevel.py      #
#                                                                                                                        #
##########################################################################################################################

print(os.environ['PATH'])
os.environ['PATH'] = 'D:\Anaconda3\Library\openslide-win64-20171122\\bin'+';'+os.environ['PATH']


import numpy as np
import openslide
import pandas as pd
from PIL import Image
from tqdm import tqdm
import shutil
from typing import List

from ufy.core.utils.tools import energy,find_file

def cut_img_energy(img_file, todir, file_format=['mrxs', 'tiff', 'ndpi'], energy_threshold=0.6, start_k=0, end_k=0,
                   cut_size=(2000, 2000), overlap=(0, 0), issave=False, max_level=4):
    ''' 切图，并按能量值排序
    :param img_file: 图片文件
    :param todir: 保存路径
    :param file_format: 默认支持'mrxs','tiff','npdi'格式的图片
    :param energy_threshold:能量阈值
    :param K: 取文件个数
    :param cut_size: 切图尺寸
    :param overlap: 重叠像素，默认不重叠
    :param issave: 是否保存图片，默认不保存
    :param max_level: 最大图层数，默认值为：4
    :return: count 切图并筛选的个数
    '''
    cut_size = np.array(cut_size).astype(int)
    overlap = np.array(overlap).astype(int)
    if start_k > end_k:
        print('输入必须满足：start_k < end_k')
    if os.path.basename(img_file).split('.')[-1] not in file_format:
        print('不支持该格式的文件，请检查文件格式！！！')
        print('目前支持的文件格式有：', file_format)
        return 0

    # open image
    try:
        source = openslide.open_slide(img_file)
    except:
        print("XXXXXX 不能打开文件：" + img_file)
        return 0

    level_count = source.level_count
    if level_count > max_level:
        LEVEL = max_level
    else:
        LEVEL = level_count - 1

    if (LEVEL < max_level):
        times = 2 ** LEVEL
        TILESIZE = cut_size // times
    else:
        times = 2 ** max_level
        TILESIZE = cut_size // times

    print(img_file + " 总计有:" + str(source.level_count) + "层")

    try:
        img = np.array(source.read_region((0, 0), LEVEL, source.level_dimensions[LEVEL]).convert('RGB'))
    except:
        print("XXXXXX 不能打开文件-：" + img_file)
        return 0

    eng = energy(img)
    w, h = eng.shape
    num_pixes = TILESIZE[0] * TILESIZE[0]

    # ROI grid
    grid = []
    power = []
    count = 0
    for i in np.arange((w - overlap[0]) // (TILESIZE[0] - overlap[0])):
        x = i * (TILESIZE[0] - overlap[0])
        for j in np.arange((h - overlap[1]) // (TILESIZE[1] - overlap[1])):
            y = j * (TILESIZE[1] - overlap[1])
            tile_eng = eng[x:(x + TILESIZE[0]), y:(y + TILESIZE[1])].copy()
            tile_power = np.sum(tile_eng) / num_pixes
            power.append(tile_power)
            grid.append([x, y])

    power_grid = pd.DataFrame({'power': power, 'grid': grid})
    power_grid = power_grid.sort_values(by='power', ascending=False)
    if not os.path.exists(todir):
        os.makedirs(todir)
    if energy_threshold == 0:
        try:
            with tqdm(range(start_k, end_k + 1, 1)) as t:
                for i in t:
                    if i < len(power_grid):
                        count += 1
                        if issave:
                            loc = power_grid.iloc[i]
                            x = loc['grid'][0] * times
                            y = loc['grid'][1] * times
                            filename = todir + '/' + str(i + 1) + '__' + np.str(x) + '_' + np.str(y) + '.jpg'
                            image = np.array(source.read_region((y, x), 0, (2000, 2000)))  # (shape : 2000,2000,4)
                            image = Image.fromarray(image).convert(mode='RGB')
                            image.save(filename)
        except:
            t.close()
            raise
        t.close()
        return count
    else:
        try:
            with tqdm(range(len(power_grid))) as t:
                for i in t:
                    loc = power_grid.iloc[i]
                    energy_tile = loc['power']
                    if energy_tile > energy_threshold:
                        count += 1
                        if issave:
                            x = loc['grid'][0] * times
                            y = loc['grid'][1] * times
                            filename = todir + '/' + str(i + 1) + '__' + np.str(x) + '_' + np.str(y) + '.jpg'
                            image = np.array(source.read_region((y, x), 0, size=cut_size))  # (shape : 2000,2000,4)
                            image = Image.fromarray(image).convert(mode='RGB')
                            image.save(filename)
                    else:
                        break
        except KeyboardInterrupt:
            t.close()
            raise
        t.close()
        return count


def cut_img_dir(dir_source='', todir_pre='', file_format=['mrxs', 'tiff', 'ndpi'], energy_threshold=0.6, start_k=0,
                end_k=0, cut_size=(2000, 2000), overlap=(0, 0), issave=False, max_level=4):
    '''
    :param dir_source:
    :param todir_pre:
    :param file_format:
    :param energy_threshold:
    :param start_k:
    :param end_k:
    :param cut_size:
    :param overlap:
    :param issave:
    :param max_level:
    :return:
    '''

    temp_dir = '../data/temp/'  # 临时文件夹
    img_files = find_file(dir_source=dir_source, file_format=file_format)
    print('图片文件数：', len(img_files))

    count = 0
    for img_file in img_files:
        if not os.path.exists(temp_dir):
            os.makedirs(temp_dir)

        if not copy_file_to_tempdir(filename=img_file, copydir_format=['mrxs'], temp_dir=temp_dir):
            continue
        todir = make_todir(filename=img_file, todir_pre=todir_pre, dir_pre=dir_source)
        count += cut_img_energy(img_file, todir=todir, file_format=file_format,
                                energy_threshold=energy_threshold, cut_size=cut_size, overlap=overlap,
                                issave=issave, max_level=max_level, start_k=start_k, end_k=end_k)
        shutil.rmtree(temp_dir)
    print('tiles 总数：', count)
    return count


def copy_file_to_tempdir(filename: str, copydir_format: List[str] = ['mrxs'], temp_dir: str = 'data/temp/') -> bool:
    '''
    :param filename: 待copy的文件名
    :param copydir_format: 需要copy文件夹的文件格式
    :param temp_dir: 临时保存的文件夹
    :return: 返回bool值，表示是copy成功
    '''
    shutil.copyfile(filename, temp_dir + os.path.basename(filename))
    filetype = os.path.basename(filename).split('.')[-1]
    if filetype in copydir_format:
        f_dir = filename.replace(filetype, '')
        if not os.path.exists(f_dir):
            print(filename, '缺少相关文件！！！')
            return False
        flist = os.listdir(f_dir)
        temp_dir2 = os.path.join(temp_dir, os.path.basename(filename).split('.')[0] + '/')
        if not os.path.exists(temp_dir2):
            os.makedirs(temp_dir2)
        for f_ in flist:
            shutil.copyfile(os.path.join(f_dir, f_), temp_dir2 + os.path.basename(f_))
    return True


def make_todir(filename: str, dir_pre: str, todir_pre: str) -> str:
    '''
    :param filename: 文件名
    :param dir_pre: 文件名前缀，用于去掉无用前缀(共同目录部分)
    :param todir_pre: todir的根目录
    :return:
    '''
    f = filename.replace(dir_pre, '')  # 去掉文件路径无用的头信息
    todir_tail = f.split('.')[0] + '/'  # 保存子子文件夹的名字
    todir = os.path.join(todir_pre, todir_tail)
    try:
        if not os.path.isdir(todir):
            os.makedirs(todir)
    except:
        print('创建文件夹失败！')
    return todir


if __name__ == '__main__':
    # cut_img_dir(dir_source="K:\\raw-slices\\huaxi-raw slices\\12\\12-raw slices\\12  HE-3d\\zz4-无癌\\", todir_pre='L:/12-normal-tiles/', file_format=['mrxs', 'tiff', 'npdi'],
    #             issave=True, start_k=100, end_k=600, energy_threshold=0)
    # cut_img_dir(dir_source="../data/original_dir/", todir_pre='../data/test/', file_format=['mrxs', 'tiff', 'npdi'],
    #             issave=True, start_k=100, end_k=600, energy_threshold=0)
    cut_img_dir(dir_source="E:\OneDrive\\rebar\一期善后\stitich\\", todir_pre='../data/test/',
                file_format=['mrxs', 'tiff', 'npdi', 'bmp'],
                issave=True, start_k=0, end_k=600, energy_threshold=0, overlap=(500, 500))
